	nam SuperOric  V1.46 by FF
	org $008000
	cou 2

; excution de la rom Oric en mode natif 65816 (mais avec les registres sur 8 bits bien sr).
; (l'excution en mode mulation est un problme pour les handlers d'interruption:
; en mode mulation, la banque programme n'est pas empile avec l'adresse de retour sur la pile,
; donc on ne sait pas dans quelle banque se trouvait le programme interrompu et le rti reste
; dans la banque courante, c.a.d 0. => il faudrait que tout le code soit en banque 0, ce qui
; empche de mettre du code en ram (banque 7E et 7F);  rtudier peut-tre avec une excution
; directe en rom (difficile  cause des accs aux donnes en banque programme)).
;
; Problme en mode natif: S est sur 16 bits et TXS charge aussi le poids fort de X,
; qui vaut 0 quand le registre X est sur 8 bits.
; Solution choisie ici: remplacer les TXS par des TCS, en gardant toujours la valeur 1 dans le
; poids fort de l'accumulateur (dans B donc). Attention donc lors des passages en 16 bits !

Basic					; 16 Ko donc
	bin basic2.rom

	jmp indirect_start	;  la fin pour vrifier qu'il n'y a pas de dcalage de code
	jmp nmi_off
	jmp nmi_on
	jmp init_charset
	jmp hires_transfer


start
	clc
	xce				; mode natif 65816
	stz $030E		; empche les interruptions "VIA"
	jsr init_ppu
	jsr clear_vram
	jsr clear_ram
	jsr dma_screen
	jsr init_graphscreen
	jsr init_palette
	jsr dma_palette
	jsr init_sound

	sep #$30		; A,X,Y 8 bits
	lda #$0F
	sta $2100		; screen on, max brightness
	lda #$A1		; autorise NMI/Vblank interrupt, IRQ (Vertical counter) et lecture automatique joysticks
	sta $4200

	lda #$7E
	pha
	plb				; Data Bank = 7E

	jsr copyrom


	lda #$01
	xba				; afin que les TCS laissent S en page 1
	lda #0
	tax
	tay
	jmp	$7EF88F		; cold reset Basic Oric

; ***********************************************************

init_graphscreen
	php
	rep #$10		; X,Y 16 bits
	sep #$20		; Accu 8 bits
	ldx.w #$7C00	; initialisation BG1
	stx $2116
	ldx.w #64
loop_bg1
	stx $2118
	inx
	cpx.w #960		; 64+28*32
	bne loop_bg1
	plp
	rts
; ***********************************************************

clear_ram
	php
	rep #$10		; X,Y 16 bits
	sep #$20		; Accu 8 bits

	ldx.w #$0300	; vite d'effacer la pile, et le vecteur d'indirection NMI !!
	txa
loop_ram1
	sta.l $7e0000,x
	inx
	bne loop_ram1
loop_ram2
	sta.l $7f0000,x
	inx
	bne loop_ram2

	rep #$30		; A,X,Y 16 bits
	ldx.w #0
	lda.w #$00FF
loop_ram4
	sta.l $7FF000,x
	inx
	inx
	inx
	inx
	cpx.w #$0800
	bne loop_ram4

	sep #$20
	ldx.w #$FF00
	lda #$FF
loop_ram3			; marque toute la ram graphique dirty
	sta.l $7f0000,x
	inx
	bne loop_ram3
	plp
	rts

; ***********************************************************

copyrom			; transfert de la rom Oric
	php
	rep #$10		; X,Y 16 bits
	sep #$20		; Accu 8 bits

	ldx.w #0
romloop
	lda.l Basic,x
	sta $C000,x
	inx
	cpx.w #$4000
	bne romloop

	lda.l $3FFFF	; pas de patch CLOAD si le dernier octet de flash est vierge
				; 128 Ko: 4 banques 32K, donc dernier octet en 3FFFF
	cmp #$FF
	beq end_copy

	lda #$4C		; patch pour un bootstrap CLOAD
	sta $ED83
	ldx.w #$E738
	stx $ED84
end_copy
	plp
	rts

; ***********************************************************
; handlers d'interruption:
;
; attention, il faut passer sur 16 bits avant de sauver les
; registres, sinon il se pourrait qu'on ne sauve que des registres
; 8 bits et qu'on perde la partie haute en passant ensuite sur 16 bits.

nmi_handler
	rep #$30		; A,X,Y sur 16 bits
	pha				; pour sauver les registres complets
	phx
	phy
	phb				; sauve DB aussi, DP non trait

	sep #$30		; A,X,Y sur 8 bits
	phk
	plb				; initialise DB  0 pour accder aux IOs
	ldx $4210		; reset NMI interrupt
;	stz $210f
;	stz $210f

	jsr dma_palette
	jsr dma_charset
	jsr dma_screen
;	jsr dma_graphic		;pas le temps de faire grand chose en mode NTSC
	jsr read_kbd
int_return
	plb				; rcupre DB
	rep #$30		; A,X,Y sur 16 bits
	ply				; pour rcuprer les registres complets
	plx
	pla
	rti				; la rcupration de P redonnera la bonne taille aux registres

irq_handler
	rep #$30		; A,X,Y sur 16 bits
	pha				; pour sauver les registres complets
	phx
	phy
	phb				; sauve DB aussi, DP non trait

	sep #$30		; A,X,Y sur 8 bits pour excuter la routine d'interruption Oric
	phk
	plb				; initialise DB  0 pour accder aux IOs
	lda $4211		; reset IRQ

	lda #$7E		; passe  la banque Oric
	pha
	plb

	lda #$C0
	sta $030D		; simule une interruption Timer 1
	bit $030E		; est-ce que le VIA mul laisse passer les interruptions ?
	bpl int_return	; non, pas d'interruption... 
					; (pb, la ractivation des interruptions dans l'IER ne redclenchera
					;  pas immdiatement une interruption, il faudra attendre la trame
					;  suivante)

	phk				; empile une adresse de retour
	pea int_return
	php				; et des drapeaux
	jmp.l $7E0244

abort_handler
	rti				; utilisable sur la SNES ?

brk_handler
	php
	rep #$10		; X,Y sur 16 bits
	sep #$20		; et A sur 8 bits
	phb
	pha
	lda 7,s			; banque programme de l'adresse de retour
	pha
	plb
	rep #$30		; A,X,Y sur 16 bits
	lda 5,s
	tax				; adr de retour du BRK dans X
	sep #$20		; A sur 8 bits
	lda.w $FFFF,x	; numro d'appel systme
	asl
	tax
	sep #$10		; X,Y sur 8 bits
	rep #$10		; X,Y sur 16 bits
	pla
	plb
	plp
	jsr (system_calls,x)
	rti


; ***********************************************************
; initialisation de la VRAM
clear_vram
	php
	rep #$30	; A,X,Y sur 16 bits
	ldx.w #$0000
	stx $2116
	lda.w #0
vram_init
	sta $2118
	inx
	bpl vram_init
	plp
	rts

; ***********************************************************

; initialisation des couleurs
init_palette
	php
	sep #$30		; A,X,Y sur 8 bits
	ldx #0
	lda #0
init_colors
	sta.l $7FF800,x
	sta.l $7FF900,x
	inx
	bne init_colors
	lda #$FF
	sta.l $7FF802
	sta.l $7FF803
	plp
	rts

; ***********************************************************

; transfert de caractres dynamiques par DMA
dma_charset
	php
	sep	#$30		; A,X,Y sur 8 bits
	ldx	#$0F
dmacharsetloop
	lda.l	$7FFF00,x
	bne	chardirty
	dex
	bpl	dmacharsetloop
	plp
	rts
chardirty
	lda #$80		; VRAM address increased after data written to $2119 (=> word VRAM access)
	sta $2115
	lda #1
	sta $4300		; criture d'un mot  chaque fois
	lda #$18
	sta $4301		; en VRAM ($2118)

	stz $4302		; adresse des tuiles en RAM
	stx $4303
	lda #$7F		; et banque
	sta $4304
	lda #1		; nb d'octets  transfrer (high)
	stz $4305
	sta $4306

	txa
	lsr
	sta $2117		; VRAM address initialisation
	lda #0
	ror
	sta $2116

	lda #$01		; DMA 1 enable
	sta $420B

	lda	#0
	sta.l $7FFF00,x
	plp
	rts

; ***********************************************************
; transfert par DMA de la palette
dma_palette
	php
	rep #$30		; A sur 16 bits
	sep #$10		; X,Y sur 8 bits

	lda.l	$7FFFF8
	beq	palettenotdirty
	lda.w	#0
	sta.l	$7FFFF8

	ldx #0
	stx $2121		; commencer  la couleur 0

	stx $4300		; criture d'un octet  chaque fois
	ldx #$22
	stx $4301		; en CGRAM ($2122)
	lda.w #$F800		; adresse de la palette en RAM
	sta $4302
	ldx #$7F		; et la banque
	stx $4304
	lda.w #512		; nb d'octets  transfrer
	sta $4305
	ldx #$01		; DMA 1 enable
	stx $420B
palettenotdirty
	plp
	rts
; ***********************************************************
; transfert par DMA du BG texte
dma_screen
	php
	rep #$30		; A sur 16 bits
	sep #$10		; X,Y sur 8 bits

	ldx #$80
	stx $2115

	lda.w #$7800
	sta $2116		; address for VRAM read/write
	ldx #1
	stx $4300		; criture d'1 word  chaque fois ($2118)
	ldx #$18
	stx $4301		; , de cpumem vers $2118 (vram)
	lda.w #$F000
	sta $4302		; adresse source
	ldx #$7F		; banque adresse source
	stx $4304
	lda.w #2048		; nb octets  transfrer
	sta $4305
	ldx #$01        
	stx $420B		; enable DMA transfer
	plp
	rts

; ***********************************************************
; transfert par DMA de pages dirty
; (c.a.d de lignes de l'cran graphique, ou de dfinitions caractres)
; 
dma_graphic
	php
	sep	#$30	; A,X,Y sur 8 bits
	ldy	#10	; nombre max de couples de lignes transfrables
	ldx	#$10
graphicdmaloop
	lda.l	$7FFF00,x
	ora.l	$7FFF01,x
	beq	linenotdirty

	lda	$021F		; flag HIRES
	beq	setformation
	lda	#$0C		; graphics formation 8 bits
setformation
	ora	#$80		; word access for VRAM
	sta	$2115
	lda	#1
	sta	$4300
	lda	#$18
	sta	$4301
	lda	#$7F	; banque adresse source
	sta	$4304

	txa
	stz	$4302	; adresse source
	sta	$4303
	lsr		; divise par 2 pour avoir une adresse mot
	stz	$2116	; adresse destination
	sta	$2117	; adresse destination en VRAM (high)
	lda	#2
	stz	$4305	; nb d'octets : 512
	sta	$4306
		; DMA 1 enable
	lda	#1
	sta	$420B

	txa
	lsr
	cmp	$2117
	beq	endgraphicdmaloop	; si le DMA n'est pas all au bout, on abandonne
	lda	#0
	sta.l	$7FFF00,x
	sta.l	$7FFF01,x

	dey
	beq	endgraphicdmaloop
linenotdirty
	inx
	inx
	cpx	#$F0
	bne	graphicdmaloop
endgraphicdmaloop
	plp
	rts

;
hires_transfer
	php
	phb
	phk			; change DB pour accder aux registres IO
	plb
	sep	#$30		; A,X,Y sur 8 bits
	lda	#$80		; screen OFF
	sta	$2100
	lda	#$8C		; graphics formation 8 bits + word access for VRAM
	sta	$2115
	lda	#1
	sta	$4300
	lda	#$18
	sta	$4301
	lda	#$7F		; banque adresse source
	sta	$4304

	stz	$4302		; adresse source
	lda	#$10
	sta	$4303
	lsr			; divise par 2 pour avoir une adresse mot
	stz	$2116		; adresse destination
	sta	$2117		; adresse destination en VRAM (high)
	lda	#$E0
	stz	$4305		; nb d'octets : E000
	sta	$4306

	lda	#1
	sta	$420B		; DMA 1 enable

	lda	#$0F		; screen ON
	sta	$2100
	plb
	plp
	rtl

; ***********************************************************
;
; Les modes 3 et 4 sont utiliss pour bnficier d'un Background 1 (BG1) en Direct Color 256,
; qui donnera un cran HIRES en dfinissant 28x32=896 tuiles.
; En mode 3 (accessible par LORES 1), les tuiles du BG2 sont en 16 couleurs, 
; seulement 128 tuiles pourront tre dfinies sans interfrer avec l'cran HIRES.
; Par dfaut, on utilise le mode 4 (LORES 0), les tuiles du BG2 sont en 4 couleurs et
; 256 tuiles peuvent tre dfinies sans dborder sur l'cran HIRES.
; Mais attention, en mode 4, le dbut du BG3 fournit des dfinitions Offset Change pour les
; 32 colonnes de tuiles des BG1 et BG2 : comme toute la VRAM est utilise pour les BG1 et BG2,
; on place BG3  la mme adresse que BG1 mais avec un Vertical Offset de 240 lignes...
;
; La carte mmoire de la VRAM est alors la suivante (en adresses mot) :
;
; 0000-07ff Dfinitions des tuiles BG2 (caractres redfinissables)
; 0800-77ff Dfinitions des tuiles BG1 (HIRES data)
; 7800-7bff BG2
; 7C00-7fff BG1 (avec dbut BG3 en 7fe0)
; 

init_ppu
	php
	phd
	rep #$30	; A,X,Y sur 16 bits
	lda.w #$2100
	tcd			; DP en $2100 pour accder aisment aux IOs
	sep #$20	; A sur 8 bits

	lda #$80
	sta $00		; screen off

	stz $01		; OBSEL
	lda #4
	sta $05		; MODE 4 : BG2 tiles en 4 couleurs
	stz $06		; mosaic off

	lda #$7C	; BG1 = VRAM $7C00-7fff (32x28)
	sta $07
	sta $09		; BG3 idem
	lda #$78	; BG2 = VRAM $7800-7bff (32x28)
	sta $08
	lda #$00	; BG1 tiles = $0800-$77ff, BG2 tiles = $0000-$07ff (128 tiles)
	sta $0b		; on utilise les tuiles BG1  partir de 64 pour commencer en $800

	lda #$FF
	stz $0d
	stz $0d		; 0 dans BG1HOFS
	sta $0e
	sta $0e		; -1 dans BG1VOFS
	stz $0f
	stz $0f		; 0 dans BG2HOFS
	sta $10
	sta $10		; et dans BG2VOFS
	stz $11
	stz $11		; 0 dans BG3HOFS
	lda #$F0
	sta $12
	stz $12		; 240 dans BG3VOFS

	lda #$80	; accs word pour la VRAM
	sta $15

	stz $23		; window off
	stz $24
	stz $25

	lda #2		; enable BG2
	sta $2c		; in main screen
	stz $2d		; no sub screen
	stz $2e		; window mask in main screen
	stz $2f		; window mask in sub screen

	stz $30		; no color addition
	stz $31
	stz $32

	stz $33		; SETINI

	stz $4201	; port I/O (signification des bits ?)
	ldx.w #150	; V-Count IRQ  la 150e ligne
	stx $4209
	stz $420B	; pas de DMA en cours
	stz $420C	; ni de HDMA
	lda #1
	sta $420D	; 3.58 MHz access cycle
	pld
	plp
	rts

;init_charset	; initialisation charset faite par la rom Oric
;			; on se contente d'initialiser les caractres
;			; 128  255 pour le clignotement curseur
;			; (mode LORES 0 seulement)
;	lda $0221
;	bne init_charset_end
;	lda #$FF
;cursor_loop
;	sta.l $7F0800,x
;	sta.l $7F0900,x
;	sta.l $7F0A00,x
;	sta.l $7F0B00,x
;	sta.l $7F0C00,x
;	sta.l $7F0D00,x
;	sta.l $7F0E00,x
;	sta.l $7F0F00,x
;	inx
;	inx
;	bne cursor_loop
;init_charset_end
;	rts

init_charset
	php
	phb
	phk			; source en banque programme
	plb	
	rep #$30		; A,X,Y sur 16 bits
	ldx.w #$1000
	lda.w #$00FF	; pour que les caractres 128  255 jouent le rle de curseur
	bit $0221		; ... uniquement si mode_gap==16
	beq charset_clear
	lda.w #0
charset_clear
	dex
	dex
	sta.l $7F0000,x
	bpl charset_clear
	lda.w	#$0100	; attention  ne pas changer le poids fort de A dans les appels systmes !

	ldx.w #Oric_font
	stx $0E

	sep #$30		; A,X,Y sur 8 bits
	lda #$7F		; destination en banque $7F
	sta $12
	lda #0
	sta $10
	lda $0221	; mode_gap
	lsr
	lsr
	lsr
	adc #2
	sta $11

	ldx #3		; 3 pages de dfinitions de caractres  transfrer
	stx $33
	ldy #0
charloop1
	ldx #8
charloop2
	lda ($0E),y
	iny
	sta [$10]
	inc $10
	inc $10
	dex
	bne charloop2
	clc
	lda $10
	adc $0221
	sta $10
	bne *+4
	inc $11
	cpy #0
	bne charloop1
	inc $0F
	dec $33
	bne charloop1

	ldx #$0F		; marque dirty le nouveau charset
	lda #$FF
charloop3
	sta.l $7FFF00,x
	dex
	bpl charloop3
	plb
	plp
	rtl			; retour long d'appel systme

;init_charset_16_colors
;	php
;	rep #$30	; X,Y sur 16 bits
;	sep #$20	; A sur 8 bits
;	ldx.w #0
;	ldy.w #0
;charset16_loop
;	lda Basic+$3C78,y
;	sta.l $7F0200,x
;	inx
;	lda #0
;	sta.l $7F0200,x
;	inx
;	sta.l $7F0200,x
;	inx
;	sta.l $7F0200,x
;	inx
;	iny
;	cpy.w #$0300
;	bne charset16_loop
;	plp
;	rts
;
;init_charset_4_colors
;	php
;	rep #$30	; X,Y sur 16 bits
;	sep #$20	; A sur 8 bits
;	ldx.w #0
;	ldy.w #0
;charset4_loop
;	lda Basic+$3C78,y
;	sta.l $7F0200,x
;	eor #$ff
;	sta.l $7F0A00,x
;	inx
;	lda #0
;	sta.l $7F0200,x
;	sta.l $7F0A00,x
;	inx
;	iny
;	cpy.w #$0300
;	bne charset4_loop
;	plp
;	rts
;


;***********************************************************
system_calls
	dcw	brk_error
	dcw	nmi_off
	dcw	nmi_on
	dcw	init_charset
	dcw	hires_transfer


brk_error
	sep #$30
	lda #$7E
	pha
	plb
	lda #1
	xba
	jmp.l $7EC496

nmi_off
	sep #$20		; A sur 8 bits
	lda #0			; interrupts disabled
	sta.l $804200
	rtl


nmi_on
	sep #$20		; A sur 8 bits
	lda #$A1		; autorise NMI/Vblank interrupt, IRQ (Vertical counter) et lecture automatique joysticks
	sta.l $804200
	rtl



; ***********************************************************
read_kbd
	php
	sep	#$30		; A,X,Y sur 8 bits

	lda	#1
	sta	$4016		; reset pulse
	jsr	tempo		; 12 cycles
	lda	#0
	sta	$4016
	jsr	tempo2		; 24 cycles = 6.3 s

	ldx	#0
loopligne
	ldy	#8
loopcol
	jsr	tempo3		; 48 cycles = 13.7 s
	lda	$4017		; + 15 cycles dans la boucle = 18 s
	lsr
	ror	$0320,x
	dey
	bne	loopcol
	lda	$0320,x		; ajustement du dcalage
	asl
	rol	$0320,x
	inx
	cpx	#8
	bne	loopligne

	plp
	rts

tempo3			; 42 cycles
	jsr tempo2
tempo2			; 18 cycles
	jsr tempo
tempo			; 6 cycles
	rts
; ***********************************************************
init_sound
	php
	phd
	pea $2100
	pld

	sep #$30		; A,X,Y sur 8 bits
	lda #2
	stz $42
	sta $43
	ldy #$CC
	sty $41
	jsr send_apu_byte
	ldx #0
	ldy #0
apu_xfr_loop
	lda.l dir,x
	sta $41
	jsr send_apu_byte
	iny
	inx
	bne apu_xfr_loop


	lda #$F2		; transfert dans les ports d'accs au DSP
	sta $42
	stz $43

	ldx #$0C		; main volume (left)
	lda #127
	jsr dsp_write
	ldx #$1C		; main volume (right)
	lda #127
	jsr dsp_write
	ldx #$5D		; directory page
	lda #2		; => directory in $0200
	jsr dsp_write
	ldx #$6C		; flags
	lda #$20		; no echo
	jsr dsp_write

	pld
	plp
	rts


dsp_write
	ldy #$55		; valeur arbitraire qui n'est pas l'offset attendu, et la soustraction index-$55 est ngative
	sty $41
	jsr send_apu_byte
	stx $41
	ldy #0
	jsr send_apu_byte
	sta $41
	iny
send_apu_byte
	sty $40
wait_ack
	cpy $40
	bne wait_ack
	rts


dir
	dcw $0220+9*0,$0220+9*0
	dcw $0220+9*2,$0220+9*2
	dcw $0220+9*4,$0220+9*4
	dcw $0220+9*6,$0220+9*6
	dcw $0220+9*8,$0220+9*9
	dcw $0220+9*11,$0220+9*12
	dcw 0,0
	dcw 0,0

; chaque forme de son est dcrite par une periode sur 32 samples,
; donc la frquence nominale  32kHz est 1000 Hz

square
	dcb	$C2,$77,$77,$77,$77,$77,$77,$77,$77
	dcb	$C3,$88,$88,$88,$88,$88,$88,$88,$88
triangle
	dcb	$C2,$01,$23,$45,$67,$76,$54,$32,$10
	dcb	$C3,$FE,$DC,$BA,$98,$89,$AB,$CD,$EF
sawtooth1
	dcb	$C2,$77,$66,$55,$44,$33,$22,$11,$00
	dcb	$C3,$FF,$EE,$DD,$CC,$BB,$AA,$99,$88
sawtooth2
	dcb	$C2,$88,$99,$AA,$BB,$CC,$DD,$EE,$FF
	dcb	$C3,$00,$11,$22,$33,$44,$55,$66,$77
sinusoid
	dcb	$C2,$00,$00,$00,$01,$23,$45,$67,$77
	dcb	$6A,$DC,$BB,$AA,$AA,$BB,$CD,$EF,$02
	dcb	$6B,$34,$55,$66,$66,$55,$43,$32,$0E
accordmaj
	dcb	$C2,$00,$00,$00,$01,$23,$45,$67,$77
	dcb	$7A,$A9,$99,$AC,$DF,$12,$45,$66,$65
	dcb	$7A,$43,$10,$FD,$DC,$CC,$CD,$EF,$01
	dcb	$6A,$56,$66,$64,$31,$FD,$CB,$AA,$BC
	dcb	$6A,$DF,$02,$45,$66,$65,$42,$1F,$DC
	dcb	$6A,$BB,$BC,$DE,$F1,$23,$44,$43,$21
	dcb	$7A,$0F,$ED,$DD,$DE,$F0,$12,$34,$45
	dcb	$7A,$54,$32,$1F,$EC,$BA,$AA,$AB,$CE
	dcb	$7B,$02,$35,$67,$77,$65,$42,$0E,$CB



; ***********************************************************
Oric_font
	; char $20 : <SPACE>  
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000

	; char $21 : !
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %000000
	dcb %001000
	dcb %000000

	; char $22 : "
	dcb %010100
	dcb %010100
	dcb %010100
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000

	; char $23 : #
	dcb %010100
	dcb %010100
	dcb %111110
	dcb %010100
	dcb %111110
	dcb %010100
	dcb %010100
	dcb %000000

	; char $24 : $
	dcb %001000
	dcb %011110
	dcb %101000
	dcb %011100
	dcb %001010
	dcb %111100
	dcb %001000
	dcb %000000

	; char $25 : %
	dcb %110000
	dcb %110010
	dcb %000100
	dcb %001000
	dcb %010000
	dcb %100110
	dcb %000110
	dcb %000000

	; char $26 : &
	dcb %010000
	dcb %101000
	dcb %101000
	dcb %010000
	dcb %101010
	dcb %100100
	dcb %011010
	dcb %000000

	; char $27 : '
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000

	; char $28 : (
	dcb %001000
	dcb %010000
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %010000
	dcb %001000
	dcb %000000

	; char $29 : )
	dcb %001000
	dcb %000100
	dcb %000010
	dcb %000010
	dcb %000010
	dcb %000100
	dcb %001000
	dcb %000000

	; char $2a : *
	dcb %001000
	dcb %101010
	dcb %011100
	dcb %001000
	dcb %011100
	dcb %101010
	dcb %001000
	dcb %000000

	; char $2b : +
	dcb %000000
	dcb %001000
	dcb %001000
	dcb %111110
	dcb %001000
	dcb %001000
	dcb %000000
	dcb %000000

	; char $2c : ,
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %001000
	dcb %001000
	dcb %010000

	; char $2d : -
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %111110
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000

	; char $2e : .
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000000
	dcb %000100
	dcb %000000
	dcb %000000

	; char $2f : /
	dcb %000000
	dcb %000010
	dcb %000100
	dcb %001000
	dcb %010000
	dcb %100000
	dcb %000000
	dcb %000000

	; char $30 : 0
	dcb %011100
	dcb %100010
	dcb %100110
	dcb %101010
	dcb %110010
	dcb %100010
	dcb %011100
	dcb %000000

	; char $31 : 1
	dcb %001000
	dcb %011000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %011100
	dcb %000000

	; char $32 : 2
	dcb %011100
	dcb %100010
	dcb %000010
	dcb %000100
	dcb %001000
	dcb %010000
	dcb %111110
	dcb %000000

	; char $33 : 3
	dcb %111110
	dcb %000010
	dcb %000100
	dcb %001100
	dcb %000010
	dcb %100010
	dcb %011100
	dcb %000000

	; char $34 : 4
	dcb %000100
	dcb %001100
	dcb %010100
	dcb %100100
	dcb %111110
	dcb %000100
	dcb %000100
	dcb %000000

	; char $35 : 5
	dcb %111110
	dcb %100000
	dcb %111100
	dcb %000010
	dcb %000010
	dcb %100010
	dcb %011100
	dcb %000000

	; char $36 : 6
	dcb %001100
	dcb %010000
	dcb %100000
	dcb %111100
	dcb %100010
	dcb %100010
	dcb %011100
	dcb %000000

	; char $37 : 7
	dcb %111110
	dcb %000010
	dcb %000100
	dcb %001000
	dcb %010000
	dcb %010000
	dcb %010000
	dcb %000000

	; char $38 : 8
	dcb %011100
	dcb %100010
	dcb %100010
	dcb %011100
	dcb %100010
	dcb %100010
	dcb %011100
	dcb %000000

	; char $39 : 9
	dcb %011100
	dcb %100010
	dcb %100010
	dcb %011110
	dcb %000010
	dcb %000100
	dcb %011000
	dcb %000000

	; char $3a : :
	dcb %000000
	dcb %000000
	dcb %001000
	dcb %000000
	dcb %000000
	dcb %001000
	dcb %000000
	dcb %000000

	; char $3b : ;
	dcb %000000
	dcb %000000
	dcb %001000
	dcb %000000
	dcb %000000
	dcb %001000
	dcb %001000
	dcb %010000

	; char $3c : <
	dcb %000100
	dcb %001000
	dcb %010000
	dcb %100000
	dcb %010000
	dcb %001000
	dcb %000100
	dcb %000000

	; char $3d : =
	dcb %000000
	dcb %000000
	dcb %111110
	dcb %000000
	dcb %111110
	dcb %000000
	dcb %000000
	dcb %000000

	; char $3e : >
	dcb %010000
	dcb %001000
	dcb %000100
	dcb %000010
	dcb %000100
	dcb %001000
	dcb %010000
	dcb %000000

	; char $3f : ?
	dcb %011100
	dcb %100010
	dcb %000100
	dcb %001000
	dcb %001000
	dcb %000000
	dcb %001000
	dcb %000000

	; char $40 : @
	dcb %011100
	dcb %100010
	dcb %101010
	dcb %101110
	dcb %101100
	dcb %100000
	dcb %011110
	dcb %000000

	; char $41 : A
	dcb %001000
	dcb %010100
	dcb %100010
	dcb %100010
	dcb %111110
	dcb %100010
	dcb %100010
	dcb %000000

	; char $42 : B
	dcb %111100
	dcb %100010
	dcb %100010
	dcb %111100
	dcb %100010
	dcb %100010
	dcb %111100
	dcb %000000

	; char $43 : C
	dcb %011100
	dcb %100010
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %100010
	dcb %011100
	dcb %000000

	; char $44 : D
	dcb %111100
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %111100
	dcb %000000

	; char $45 : E
	dcb %111110
	dcb %100000
	dcb %100000
	dcb %111100
	dcb %100000
	dcb %100000
	dcb %111110
	dcb %000000

	; char $46 : F
	dcb %111110
	dcb %100000
	dcb %100000
	dcb %111100
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %000000

	; char $47 : G
	dcb %011110
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %100110
	dcb %100010
	dcb %011110
	dcb %000000

	; char $48 : H
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %111110
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %000000

	; char $49 : I
	dcb %011100
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %011100
	dcb %000000

	; char $4a : J
	dcb %000010
	dcb %000010
	dcb %000010
	dcb %000010
	dcb %000010
	dcb %100010
	dcb %011100
	dcb %000000

	; char $4b : K
	dcb %100010
	dcb %100100
	dcb %101000
	dcb %110000
	dcb %101000
	dcb %100100
	dcb %100010
	dcb %000000

	; char $4c : L
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %111110
	dcb %000000

	; char $4d : M
	dcb %100010
	dcb %110110
	dcb %101010
	dcb %101010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %000000

	; char $4e : N
	dcb %100010
	dcb %100010
	dcb %110010
	dcb %101010
	dcb %100110
	dcb %100010
	dcb %100010
	dcb %000000

	; char $4f : O
	dcb %011100
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %011100
	dcb %000000

	; char $50 : P
	dcb %111100
	dcb %100010
	dcb %100010
	dcb %111100
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %000000

	; char $51 : Q
	dcb %011100
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %101010
	dcb %100100
	dcb %011010
	dcb %000000

	; char $52 : R
	dcb %111100
	dcb %100010
	dcb %100010
	dcb %111100
	dcb %101000
	dcb %100100
	dcb %100010
	dcb %000000

	; char $53 : S
	dcb %011100
	dcb %100010
	dcb %100000
	dcb %011100
	dcb %000010
	dcb %100010
	dcb %011100
	dcb %000000

	; char $54 : T
	dcb %111110
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %000000

	; char $55 : U
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %011100
	dcb %000000

	; char $56 : V
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %010100
	dcb %001000
	dcb %000000

	; char $57 : W
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %101010
	dcb %101010
	dcb %110110
	dcb %100010
	dcb %000000

	; char $58 : X
	dcb %100010
	dcb %100010
	dcb %010100
	dcb %001000
	dcb %010100
	dcb %100010
	dcb %100010
	dcb %000000

	; char $59 : Y
	dcb %100010
	dcb %100010
	dcb %010100
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %000000

	; char $5a : Z
	dcb %111110
	dcb %000010
	dcb %000100
	dcb %001000
	dcb %010000
	dcb %100000
	dcb %111110
	dcb %000000

	; char $5b : [
	dcb %011110
	dcb %010000
	dcb %010000
	dcb %010000
	dcb %010000
	dcb %010000
	dcb %011110
	dcb %000000

	; char $5c : \
	dcb %000000
	dcb %100000
	dcb %010000
	dcb %001000
	dcb %000100
	dcb %000010
	dcb %000000
	dcb %000000

	; char $5d : ]
	dcb %111100
	dcb %000100
	dcb %000100
	dcb %000100
	dcb %000100
	dcb %000100
	dcb %111100
	dcb %000000

	; char $5e : ^
	dcb %001000
	dcb %010100
	dcb %101010
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %000000

	; char $5f : _
	dcb %001110
	dcb %010000
	dcb %010000
	dcb %010000
	dcb %111100
	dcb %010000
	dcb %111110
	dcb %000000

	; char $60 : `
	dcb %001100
	dcb %010010
	dcb %101101
	dcb %101001
	dcb %101001
	dcb %101101
	dcb %010010
	dcb %001100

	; char $61 : a
	dcb %000000
	dcb %000000
	dcb %011100
	dcb %000010
	dcb %011110
	dcb %100010
	dcb %011110
	dcb %000000

	; char $62 : b
	dcb %100000
	dcb %100000
	dcb %111100
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %111100
	dcb %000000

	; char $63 : c
	dcb %000000
	dcb %000000
	dcb %011110
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %011110
	dcb %000000

	; char $64 : d
	dcb %000010
	dcb %000010
	dcb %011110
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %011110
	dcb %000000

	; char $65 : e
	dcb %000000
	dcb %000000
	dcb %011100
	dcb %100010
	dcb %111110
	dcb %100000
	dcb %011110
	dcb %000000

	; char $66 : f
	dcb %001100
	dcb %010010
	dcb %010000
	dcb %111100
	dcb %010000
	dcb %010000
	dcb %010000
	dcb %000000

	; char $67 : g
	dcb %000000
	dcb %000000
	dcb %011100
	dcb %100010
	dcb %100010
	dcb %011110
	dcb %000010
	dcb %011100

	; char $68 : h
	dcb %100000
	dcb %100000
	dcb %111100
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %000000

	; char $69 : i
	dcb %001000
	dcb %000000
	dcb %011000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %011100
	dcb %000000

	; char $6a : j
	dcb %000100
	dcb %000000
	dcb %001100
	dcb %000100
	dcb %000100
	dcb %000100
	dcb %100100
	dcb %011000

	; char $6b : k
	dcb %100000
	dcb %100000
	dcb %100010
	dcb %100100
	dcb %111000
	dcb %100100
	dcb %100010
	dcb %000000

	; char $6c : l
	dcb %011000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %011100
	dcb %000000

	; char $6d : m
	dcb %000000
	dcb %000000
	dcb %110110
	dcb %101010
	dcb %101010
	dcb %101010
	dcb %100010
	dcb %000000

	; char $6e : n
	dcb %000000
	dcb %000000
	dcb %111100
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %000000

	; char $6f : o
	dcb %000000
	dcb %000000
	dcb %011100
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %011100
	dcb %000000

	; char $70 : p
	dcb %000000
	dcb %000000
	dcb %111100
	dcb %100010
	dcb %100010
	dcb %111100
	dcb %100000
	dcb %100000

	; char $71 : q
	dcb %000000
	dcb %000000
	dcb %011110
	dcb %100010
	dcb %100010
	dcb %011110
	dcb %000010
	dcb %000010

	; char $72 : r
	dcb %000000
	dcb %000000
	dcb %101110
	dcb %110000
	dcb %100000
	dcb %100000
	dcb %100000
	dcb %000000

	; char $73 : s
	dcb %000000
	dcb %000000
	dcb %011110
	dcb %100000
	dcb %011100
	dcb %000010
	dcb %111100
	dcb %000000

	; char $74 : t
	dcb %010000
	dcb %010000
	dcb %111100
	dcb %010000
	dcb %010000
	dcb %010010
	dcb %001100
	dcb %000000

	; char $75 : u
	dcb %000000
	dcb %000000
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %100110
	dcb %011010
	dcb %000000

	; char $76 : v
	dcb %000000
	dcb %000000
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %010100
	dcb %001000
	dcb %000000

	; char $77 : w
	dcb %000000
	dcb %000000
	dcb %100010
	dcb %100010
	dcb %101010
	dcb %101010
	dcb %110110
	dcb %000000

	; char $78 : x
	dcb %000000
	dcb %000000
	dcb %100010
	dcb %010100
	dcb %001000
	dcb %010100
	dcb %100010
	dcb %000000

	; char $79 : y
	dcb %000000
	dcb %000000
	dcb %100010
	dcb %100010
	dcb %100010
	dcb %011110
	dcb %000010
	dcb %011100

	; char $7a : z
	dcb %000000
	dcb %000000
	dcb %111110
	dcb %000100
	dcb %001000
	dcb %010000
	dcb %111110
	dcb %000000

	; char $7b : {
	dcb %001110
	dcb %011000
	dcb %011000
	dcb %110000
	dcb %011000
	dcb %011000
	dcb %001110
	dcb %000000

	; char $7c : |
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000
	dcb %001000

	; char $7d : }
	dcb %111000
	dcb %001100
	dcb %001100
	dcb %000110
	dcb %001100
	dcb %001100
	dcb %111000
	dcb %000000

	; char $7e : ~
	dcb %101010
	dcb %010101
	dcb %101010
	dcb %010101
	dcb %101010
	dcb %010101
	dcb %101010
	dcb %010101

	; char $7f : <DEL>
	dcb %111111
	dcb %111111
	dcb %111111
	dcb %111111
	dcb %111111
	dcb %111111
	dcb %111111
	dcb %111111

;*****************************************************************************
indirect_start
	jmp.l start+$800000	; excution plus rapide en banque $80

; ***********************************************************
	pad	$FFE6

	dcw	brk_handler
	dcw	abort_handler
	dcw	nmi_handler
	dcw	$C000
	dcw	irq_handler

	pad	$FFFA

	dcw	nmi_handler
	dcw	$C000
	dcw	irq_handler




